@@ -2,11 +2,18 @@
 # Copyright (c) 2001-2007 Twisted Matrix Laboratories.
 # See LICENSE for details.
 
+import struct
+
 try:
-    import Crypto
+    import Crypto.Cipher.DES3
 except ImportError:
     Crypto = None
 
+try:
+    import pyasn1
+except ImportError:
+    pyasn1 = None
+
 from twisted.conch.ssh import common, session, forwarding
 from twisted.conch import avatar, error
 from twisted.conch.test.keydata import publicRSA_openssh, privateRSA_openssh
@@ -19,7 +26,6 @@ from twisted.trial import unittest
 
 from test_recvline import LoopbackRelay
 
-import struct
 
 
 class ConchTestRealm:
@@ -72,7 +78,7 @@ class ConchTestAvatar(avatar.ConchUser):
         return 1
 
     def logout(self):
-        loggedOut = True
+        self.loggedOut = True
         for listener in self.listeners.values():
             log.msg('stopListening %s' % listener)
             listener.stopListening()
@@ -228,7 +234,7 @@ class SuperEchoTransport:
         self.proto.processEnded(failure.Failure(ProcessTerminated(0, None, None)))
 
 
-if Crypto: # stuff that needs PyCrypto to even import
+if Crypto is not None and pyasn1 is not None:
     from twisted.conch import checkers
     from twisted.conch.ssh import channel, connection, factory, keys
     from twisted.conch.ssh import transport, userauth
@@ -690,6 +696,9 @@ class SSHProtocolTestCase(unittest.TestCase):
     if not Crypto:
         skip = "can't run w/o PyCrypto"
 
+    if not pyasn1:
+        skip = "can't run w/o PyASN1"
+
     def testOurServerOurClient(self):
         """test the Conch server against the Conch client
         """
@@ -723,16 +732,46 @@ class TestSSHFactory(unittest.TestCase):
     if not Crypto:
         skip = "can't run w/o PyCrypto"
 
-    def testMultipleFactories(self):
-        f1 = factory.SSHFactory()
-        f2 = factory.SSHFactory()
+    if not pyasn1:
+        skip = "can't run w/o PyASN1"
+
+    def makeSSHFactory(self, primes=None):
+        sshFactory = factory.SSHFactory()
         gpk = lambda: {'ssh-rsa' : keys.Key(None)}
-        f1.getPrimes = lambda: None
-        f2.getPrimes = lambda: {1:(2,3)}
-        f1.getPublicKeys = f2.getPublicKeys = gpk
-        f1.getPrivateKeys = f2.getPrivateKeys = gpk
-        f1.startFactory()
-        f2.startFactory()
+        sshFactory.getPrimes = lambda: primes
+        sshFactory.getPublicKeys = sshFactory.getPrivateKeys = gpk
+        sshFactory.startFactory()
+        return sshFactory
+
+
+    def test_buildProtocol(self):
+        """
+        By default, buildProtocol() constructs an instance of
+        SSHServerTransport.
+        """
+        factory = self.makeSSHFactory()
+        protocol = factory.buildProtocol(None)
+        self.assertIsInstance(protocol, transport.SSHServerTransport)
+
+
+    def test_buildProtocolRespectsProtocol(self):
+        """
+        buildProtocol() calls 'self.protocol()' to construct a protocol
+        instance.
+        """
+        calls = []
+        def makeProtocol(*args):
+            calls.append(args)
+            return transport.SSHServerTransport()
+        factory = self.makeSSHFactory()
+        factory.protocol = makeProtocol
+        factory.buildProtocol(None)
+        self.assertEquals([()], calls)
+
+
+    def test_multipleFactories(self):
+        f1 = self.makeSSHFactory(primes=None)
+        f2 = self.makeSSHFactory(primes={1:(2,3)})
         p1 = f1.buildProtocol(None)
         p2 = f2.buildProtocol(None)
         self.failIf('diffie-hellman-group-exchange-sha1' in p1.supportedKeyExchanges,
@@ -771,6 +810,9 @@ class MPTestCase(unittest.TestCase):
     if not Crypto:
         skip = "can't run w/o PyCrypto"
 
+    if not pyasn1:
+        skip = "can't run w/o PyASN1"
+
 
     def test_getMP(self):
         """
