@@ -96,8 +96,14 @@ class SSHAgentClient(protocol.Protocol):
 
     def signData(self, blob, data):
         """
-        Blob is a public key whose private counterpart was previously added to
-        the running agent.  Return a signature for data using that key.
+        Request that the agent sign the given C{data} with the private key
+        which corresponds to the public key given by C{blob}.  The private
+        key should have been added to the agent already.
+
+        @type blob: C{str}
+        @type data: C{str}
+        @return: A L{Deferred} which fires with a signature for given data
+            created with the given key.
         """
         req = NS(blob)
         req += NS(data)
